@@ -167,6 +167,12 @@ class Subcommands(object):
                   'help': "use a specified key name for the rerun"},
                  {'flag': ["-n", "--name"],
                   'help': "use a specified run name for the rerun"},
+                 {'flag': ["-u", "--use-spot"],
+                  'help': "use spot instance for the rerun",
+                  'action': "store_true"},
+                 {'flag': ["-U", "--do-not-use-spot"],
+                  'help': "do not use spot instance for the rerun",
+                  'action': "store_true"},
                  {'flag': ["-x", "--overwrite-input-extra"],
                   'help': "overwrite input extra file if it already exists (reserved for pony)"}],
             'rerun_many':
@@ -212,6 +218,12 @@ class Subcommands(object):
                   'help': "use a specified key name for the rerun"},
                  {'flag': ["-n", "--name"],
                   'help': "use a specified run name for the rerun"},
+                 {'flag': ["-u", "--use-spot"],
+                  'help': "use spot instance for the rerun",
+                  'action': "store_true"},
+                 {'flag': ["-U", "--do-not-use-spot"],
+                  'help': "do not use spot instance for the rerun",
+                  'action': "store_true"},
                  {'flag': ["-x", "--overwrite-input-extra"],
                   'help': "overwrite input extra file if it already exists (reserved for pony)"}],
             'setup_tibanna_env':
@@ -405,20 +417,22 @@ def info(job_id):
     """prints out information about a job"""
     print(json.dumps(API().info(job_id), indent=True))
 
+
 def rerun(exec_arn=None, job_id=None, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, app_name_filter=None,
           instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None,
-          overwrite_input_extra=None, key_name=None, name=None):
+          overwrite_input_extra=None, key_name=None, name=None, use_spot=None, do_not_use_spot=None):
     """ rerun a specific job"""
     API().rerun(exec_arn=exec_arn, job_id=job_id, sfn=sfn,
                 app_name_filter=app_name_filter, instance_type=instance_type, shutdown_min=shutdown_min,
                 ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops,
-                overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name)
+                overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name,
+                use_spot=use_spot, do_not_use_spot=do_not_use_spot)
 
 
 def rerun_many(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, stopdate='13Feb2018', stophour=13,
                stopminute=0, offset=0, sleeptime=5, status='FAILED', app_name_filter=None,
                instance_type=None, shutdown_min=None, ebs_size=None, ebs_type=None, ebs_iops=None,
-               overwrite_input_extra=None, key_name=None, name=None):
+               overwrite_input_extra=None, key_name=None, name=None, use_spot=None, do_not_use_spot=None):
     """rerun all the jobs that failed after a given time point
     filtered by the time when the run failed (stopdate, stophour (24-hour format), stopminute)
     By default, stophour should be the same as your system time zone. This can be changed by setting a different offset.
@@ -435,7 +449,8 @@ def rerun_many(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, stopdate='13Feb2018', sto
                      stopminute=stopminute, offset=offset, sleeptime=sleeptime, status=status,
                      app_name_filter=app_name_filter, instance_type=instance_type, shutdown_min=shutdown_min,
                      ebs_size=ebs_size, ebs_type=ebs_type, ebs_iops=ebs_iops,
-                     overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name)
+                     overwrite_input_extra=overwrite_input_extra, key_name=key_name, name=name,
+                     use_spot=use_spot, do_not_use_spot=do_not_use_spot)
 
 
 def stat(sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, status=None, long=False, nlines=None, job_ids=None):
