@@ -4,9 +4,11 @@ import time
 import os
 import logging
 import boto3
+import botocore
 import copy
 import re
 from . import create_logger
+from datetime import datetime, timedelta
 from .utils import (
     does_key_exist,
     create_jobid
@@ -20,8 +22,10 @@ from .vars import (
     DYNAMODB_TABLE,
     DEFAULT_ROOT_EBS_SIZE,
     TIBANNA_AWSF_DIR,
-    DEFAULT_AWSF_IMAGE
+    DEFAULT_AWSF_IMAGE,
+    AWS_REGION_NAMES
 )
+from .job import Jobs
 from .exceptions import (
     MissingFieldInInputJsonException,
     MalFormattedInputJsonException,
@@ -30,7 +34,8 @@ from .exceptions import (
     EC2InstanceLimitWaitException,
     DependencyStillRunningException,
     DependencyFailedException,
-    UnsupportedCWLVersionException
+    UnsupportedCWLVersionException,
+    PricingRetrievalException
 )
 from .base import SerializableObject
 from .nnested_array import flatten, run_on_nested_arrays1
@@ -38,6 +43,7 @@ from ._version import __version__
 from Benchmark import run as B
 from Benchmark.classes import get_instance_types, instance_list
 from Benchmark.byteformat import B2GB
+
 NONSPOT_EC2_PARAM_LIST = ['TagSpecifications', 'InstanceInitiatedShutdownBehavior',
                           'MaxCount', 'MinCount', 'DisableApiTermination']
 
@@ -73,13 +79,14 @@ class UnicornInput(SerializableObject):
         """
         args = self.args
         cfg = self.cfg
+        args.fill_default()
         cfg.fill_language_options(args.language, getattr(args, 'singularity', False))
         cfg.fill_other_fields(args.app_name)
         # sanity check
         if args.app_name and args.app_name in B.app_name_function_map:
             pass  # use benchmarking
         else:
-            if not cfg.ebs_size:  # unset (set to 0)
+            if not cfg.ebs_size or cfg.ebs_size < 10:  # unset (set to 0) or <10GB
                 cfg.ebs_size = 10  # if not set by user or benchmark, just use 10GB as default
             if not cfg.EBS_optimized:  # either false or unset
                 cfg.EBS_optimized = False  # False by default so t2 instances can be used
@@ -112,8 +119,8 @@ class Args(SerializableObject):
         for field in ['output_S3_bucket']:
             if not hasattr(self, field):
                 raise MissingFieldInInputJsonException("field %s is required in args" % field)
-        if fill_default:
-            self.fill_default()
+        #if fill_default:
+        #    self.fill_default()
 
     def update(self, d):
         for k, v in d.items():
@@ -259,6 +266,13 @@ class Config(SerializableObject):
             setattr(self, k, v)
 
     def fill_default(self):
+        # use benchmark or not
+        if not hasattr(self, 'instance_type') and (not hasattr(self, 'cpu') or not hasattr(self, 'mem')):
+            self.use_benchmark = True
+        elif not hasattr(self, 'ebs_size'):
+            self.use_benchmark = True 
+        else:
+            self.use_benchmark = False
         # fill in default
         for field in ['instance_type', 'EBS_optimized', 'cpu', 'ebs_iops', 'password', 'key_name',
                       'spot_duration', 'availability_zone', 'security_group', 'subnet']:
@@ -281,10 +295,16 @@ class Config(SerializableObject):
         if not hasattr(self, 'public_postrun_json'):
             self.public_postrun_json = False
             # 4dn will use 'true' --> this will automatically be added by start_run_awsem
+        if not hasattr(self, 'encrypt_s3_upload'):
+            self.encrypt_s3_upload = False
         if not hasattr(self, 'root_ebs_size'):
             self.root_ebs_size = DEFAULT_ROOT_EBS_SIZE
         if not hasattr(self, 'awsf_image'):
             self.awsf_image = DEFAULT_AWSF_IMAGE
+        if not hasattr(self, 'mem_as_is'):  # if false, add 1GB overhead
+            self.mem_as_is = False
+        if not hasattr(self, 'ebs_size_as_is'):  # if false, add 5GB overhead
+            self.ebs_size_as_is = False
 
     def fill_internal(self):
         # fill internally-used fields (users cannot specify these fields)
@@ -319,7 +339,8 @@ class Execution(object):
         self.user_specified_ebs_size = self.cfg.ebs_size
         # get benchmark if available
         self.input_size_in_bytes = self.get_input_size_in_bytes()
-        self.benchmark = self.get_benchmarking(self.input_size_in_bytes)
+        if self.cfg.use_benchmark:
+            self.benchmark = self.get_benchmarking(self.input_size_in_bytes)
         self.init_instance_type_list()
         self.update_config_instance_type()
         self.update_config_ebs_size()
@@ -356,7 +377,11 @@ class Execution(object):
                                             'EBS_optimized': False})
         # user specified mem and cpu
         if self.cfg.mem and self.cfg.cpu:
-            list0 = get_instance_types(self.cfg.cpu, self.cfg.mem, instance_list(exclude_t=False))
+            if self.cfg.mem_as_is:
+                mem = self.cfg.mem
+            else:
+                mem = self.cfg.mem + 1
+            list0 = get_instance_types(self.cfg.cpu, mem, instance_list(exclude_t=False))
             nonredundant_list = [i for i in list0 if i['instance_type'] != instance_type]
             instance_type_dlist.extend(nonredundant_list)
         # user specifically wanted EBS_optimized instances
@@ -404,7 +429,7 @@ class Execution(object):
         except:
             return None
 
-    def auto_calculate_ebs_size(self):
+    def update_config_ebs_size(self):
         """if ebs_size is in the format of e.g. '3x', it updates the size
         to be total input size times three. If the value is lower than 10GB,
         keep 10GB"""
@@ -419,13 +444,13 @@ class Execution(object):
                 self.cfg.ebs_size = round(self.cfg.ebs_size) + 1
             else:
                 self.cfg.ebs_size = round(self.cfg.ebs_size)
-            if self.cfg.ebs_size < 10:
-                self.cfg.ebs_size = 10
-
-    def update_config_ebs_size(self):
-        self.auto_calculate_ebs_size()  # if in the format of '3x'
         if not self.user_specified_ebs_size:  # use benchmark only if not set by user
             self.cfg.ebs_size = self.benchmark['ebs_size']
+        if not self.cfg.ebs_size_as_is:
+            self.cfg.ebs_size += 5  # account for docker image size
+        if self.cfg.ebs_size < 10:
+            self.cfg.ebs_size = 10
+
 
     def get_input_size_in_bytes(self):
         input_size_in_bytes = dict()
@@ -621,12 +646,19 @@ class Execution(object):
         logger.info("jsonbody=\n" + jsonbody)
         # copy the json file to the s3 bucket
         logger.info("json_bucket = " + self.cfg.json_bucket)
+        upload_args = {
+            'Body': jsonbody.encode('utf-8'),
+            'Bucket': self.cfg.json_bucket,
+            'Key': jsonkey
+        }
+        if self.cfg.encrypt_s3_upload:
+            upload_args.update({'ServerSideEncryption': 'aws:kms'})
         try:
             s3 = boto3.client('s3')
         except Exception as e:
             raise Exception("boto3 client error: Failed to connect to s3 : %s" % str(e))
         try:
-            s3.put_object(Body=jsonbody.encode('utf-8'), Bucket=self.cfg.json_bucket, Key=jsonkey)
+            s3.put_object(**upload_args)
         except Exception as e:
             raise Exception("boto3 client error: Failed to upload run.json %s to s3: %s" % (jsonkey, str(e)))
 
@@ -709,7 +741,7 @@ class Execution(object):
         return largs
 
     def get_instance_info(self):
-        # get public IP for the instance (This may not happen immediately)
+        # get public IP and availablity zone for the instance (This may not happen immediately)
         try:
             ec2 = boto3.client('ec2')
         except Exception as e:
@@ -721,22 +753,29 @@ class Execution(object):
                 instance_desc_log = ec2.describe_instances(InstanceIds=[self.instance_id])
                 if 'PublicIpAddress' not in instance_desc_log['Reservations'][0]['Instances'][0]:
                     instance_ip = ''
+                    availability_zone = ''
                     break
                 instance_ip = instance_desc_log['Reservations'][0]['Instances'][0]['PublicIpAddress']
+                availability_zone = instance_desc_log['Reservations'][0]['Instances'][0]["Placement"]["AvailabilityZone"]
                 break
             except:
                 continue
-        return({'instance_id': self.instance_id, 'instance_ip': instance_ip, 'start_time': self.get_start_time()})
+        return({'instance_id': self.instance_id, 
+                'instance_ip': instance_ip, 
+                'availability_zone' : availability_zone,
+                'start_time': self.get_start_time()})
 
-    def check_dependency(self, exec_arn=None):
-        if exec_arn:
-            client = boto3.client('stepfunctions', region_name=AWS_REGION)
-            for arn in exec_arn:
-                res = client.describe_execution(executionArn=arn)
-                if res['status'] == 'RUNNING':
-                    raise DependencyStillRunningException("Dependency is still running: %s" % arn)
-                elif res['status'] == 'FAILED':
-                    raise DependencyFailedException("A Job that this job is dependent on failed: %s" % arn)
+    @staticmethod
+    def check_dependency(exec_arn=None, job_id=None):
+        """take a list of exec_arns and/or a list of job_ids
+        and raise DependencyFailedException if any of these failed
+        or raise DependencyStillRunningException if any of these are running.
+        """
+        job_statuses = Jobs.status(job_ids=job_id, exec_arns=exec_arn)
+        if job_statuses['failed_jobs']:
+            raise DependencyFailedException("A Job that this job is dependent on failed: %s" % ','.join(job_statuses['failed_jobs']))
+        if job_statuses['running_jobs']:
+            raise DependencyStillRunningException("Dependency is still running: %s" % ','.join(job_statuses['running_jobs']))
 
     def add_instance_id_to_dynamodb(self):
         dd = boto3.client('dynamodb')
@@ -880,8 +919,8 @@ class Execution(object):
             DashboardName=dashboard_name,
             DashboardBody=json.dumps(body)
         )
-
-
+       
+    
 def upload_workflow_to_s3(unicorn_input):
     """input is a UnicornInput object"""
     args = unicorn_input.args
@@ -905,10 +944,15 @@ def upload_workflow_to_s3(unicorn_input):
         localdir = args.cwl_directory_local
     wf_files.append(main_wf)
     localdir = localdir.rstrip('/')
+    # encrypted s3 upload
+    extra_args = {}
+    if cfg.encrypt_s3_upload:
+        extra_args.update({"ServerSideEncryption": "aws:kms"})
+    s3 = boto3.client('s3')
     for wf_file in wf_files:
         source = localdir + '/' + wf_file
         target = key_prefix + wf_file
-        boto3.client('s3').upload_file(source, bucket, target)
+        s3.upload_file(source, bucket, target, ExtraArgs=extra_args)
     url = "s3://%s/%s" % (bucket, key_prefix)
     if args.language == 'wdl':
         args.wdl_directory_url = url
